--Thanks to vhaarr for helping me out with reducing the length of code
--Thanks to pastamancer for fixing the issues with Supreme Power Flasks and pointing in right direction for others
--Window Resizing code based off the dragbar from violation
--And also thanks to all those in #wowace for the various suggestions

BigBrother = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0")

local addon = BigBrother

local AceEvent = AceLibrary("AceEvent-2.0")
local RL = AceLibrary("Roster-2.1")
local aura = AceLibrary("SpecialEvents-Aura-2.0")
local parser = ParserLib:GetInstance("1.1")
local L = AceLibrary("AceLocale-2.2"):new("BigBrother")
local BS = AceLibrary("Babble-Spell-2.2")

local buffs = {
	L["Chromatic Resistance"],
	L["Distilled Wisdom"],
	L["Flask of Fortification"],
	L["Flask of Mighty Restoration"],
	L["Flask of Relentless Assault"],
	L["Supreme Power"],
	L["Flask of the Titans"],
	L["Supreme Power of Shattrath"],
	L["Fortification of Shattrath"],
	L["Mighty Restoration of Shattrath"],
	L["Relentless Assault of Shattrath"],
	-- delete after 2.2
	L["Flask of Shadow Fortification"],
	L["Flask of Arcane Fortification"],
	-- new for 2.2
	L["Flask of Pure Death"],
	L["Flask of Blinding Light"],
	L["Chromatic Wonder"],
}

local color = "|cffff8040%s|r"
local MissingBuffList={}

BigBrotherVars={}

local options = {
	type = 'group',
	handler = BigBrother,
	args = {
		flaskcheck = {
			name = L["Flask Check"],
			desc = L["Checks for flasks on the raid."],
			type = 'group',
			args = {
				self = {
					name = L["Self"],
					desc = L["Reports result only to yourself."],
					type = 'execute',
					func = "FlaskCheck",
					passValue = "SELF",
				},
				party = {
					name = L["Party"],
					desc = L["Reports result to your party."],
					type = 'execute',
					func = "FlaskCheck",
					disabled = function() return GetNumPartyMembers()==0 end,
					passValue = "PARTY",
				},
				raid = {
					name = L["Raid"],
					desc = L["Reports result to your raid."],
					type = 'execute',
					func = "FlaskCheck",
					disabled = function() return GetNumRaidMembers()==0 end,
					passValue = "RAID",
				},
				guild = {
					name = L["Guild"],
					desc = L["Reports result to guild chat."],
					type = 'execute',
					func = "FlaskCheck",
					passValue = "GUILD",
				},
				officer = {
					name = L["Officer"],
					desc = L["Reports result to officer chat."],
					type = 'execute',
					func = "FlaskCheck",
					passValue = "OFFICER",
				}
			}
			
		},
		polymorph = {
			name  = BS["Polymorph"],
			desc = L["Reports if hostile polymorphs are broken and by who."],
			type = 'toggle',
			get = function() return BigBrotherVars.PolyBreak end,
			set = function(v)
				BigBrotherVars.PolyBreak=v 
				if v then
					parser:RegisterInfoType("BigBrother","dispel", addon.SheepBreaker)
				else
					parser:UnregisterInfoType("BigBrother","dispel")
				end
			end,
			map = { [false] = "|cffff4040Disabled|r", [true] = "|cff40ff40Enabled|r" }
		},
		misdirect = {
			name  = L["Misdirect"],
			desc = L["Reports who gains misdirection."],
			type = 'toggle',
			get = function() return BigBrotherVars.Misdirect end,
			set = function(v) BigBrotherVars.Misdirect = v end,
			map = { [false] = "|cffff4040Disabled|r", [true] = "|cff40ff40Enabled|r" }
		},
		buffcheck = {
			name = L["BuffCheck"],
			desc = L["Pops up a window to check various raid/elixir buffs (drag the bottom to resize)."],
			type = 'execute',
			func = function() CreateBuffWindow() end,
		}
	}
}

function addon:OnInitialize()
	self:RegisterChatCommand("/bb", "/bigbrother", options, "BIGBROTHER")
end

function addon:OnEnable()
	parser:RegisterInfoType("BigBrother","dispel", addon.SheepBreaker)
	parser:RegisterInfoType("BigBrother_BuffWindow","dispel", addon.BuffUpdating)
	self:RegisterEvent("SpecialEvents_UnitBuffGained")
	self:RegisterEvent("SpecialEvents_UnitBuffLost", "BuffUpdating")
	BigBrotherVars.PolyBreak = true
	BigBrotherVars.Misdirect = true
end

function addon:SpecialEvents_UnitBuffGained(unit, name, index, count, icon, rank)
	if name == BS["Misdirection"] then
		if BigBrotherVars.Misdirect then
			self:Print(L["Misdirection on |cff40ff40%s|r."]:format(UnitName(unit)))
		end
	else
		self:BuffUpdating()
	end
end

function addon:SendMessageList(Pre,List,Where)
	if #List > 0 then
		if Where == "SELF" then
			self:Print(string.format(color, Pre..":") .. " " .. table.concat(List, ", "))
		else
			SendChatMessage(Pre..": "..table.concat(List, ", "),Where)
		end
	end
end

function addon:HasBuff(player)
	for k, v in pairs(MissingBuffList) do
		if v==player then
			table.remove(MissingBuffList,k)
		end
	end
end

function addon:FlaskCheck(Where)
	local totalCount = 0

	MissingBuffList={}
	for unit in RL:IterateRoster(false) do
		table.insert(MissingBuffList,unit.name)
	end

	for i, v in ipairs(buffs) do
		local t = self:BuffPlayerList(v)
		self:SendMessageList(v, t, Where)
		totalCount = totalCount + #t
	end
	
	self:SendMessageList(L["No Flask"], MissingBuffList, Where)
end

function addon:BuffPlayerList(buffname)
	local list = {}
	for unit in RL:IterateRoster(false) do
		if aura:UnitHasBuff(unit.unitid, buffname) then
			table.insert(list, unit.name)
			self:HasBuff(unit.name)
		end
	end
	return list
end

function addon:SheepBreaker(info)
	--Since we don't register the event for when variables are loaded
	if not BigBrotherVars.PolyBreak then
		parser:UnregisterInfoType("BigBrother","dispel")
		return
	end

	local Polymorph=string.find(info.skill,BS["Polymorph"])
	if Polymorph and info.source then
		local SourceHostile,VictimHostile

		if info.source==ParserLib_SELF then
			info.source=UnitName("PLAYER")
		end
		if info.victim==ParserLib_SELF then
			info.victim=UnitName("PLAYER")
		end

		if RL:GetUnitIDFromName(info.victim) then
			--We don't care about friendlies being polymorphed
			return
		else
			VictimHostile="|cffff4040"
		end

		if RL:GetUnitIDFromName(info.source) then
			SourceHostile="|cff40ff40"
		else
			SourceHostile="|cffff4040"
		end

		if info.sourceSkill then
			BigBrother:Print(L["Polymorph on %s|r removed by %s|r's %s"]:format(VictimHostile..info.victim, SourceHostile..info.source, info.sourceSkill))
		else
			BigBrother:Print(L["Polymorph on %s|r removed by %s|r"]:format(VictimHostile..info.victim, SourceHostile..info.source))
		end
	elseif Polymorph then
		if info.victim==ParserLib_SELF then
			info.victim=UnitName("PLAYER")
		end

		if RL:GetUnitIDFromName(info.victim) then
			--We don't care about friendlies being polymorphed
			return
		else
			VictimHostile="|cffff4040"
		end

		BigBrother:Print(L["Polymorph on %s|r is removed"]:format(VictimHostile..info.victim))
	end
end

function addon:BuffUpdating(info)
	if BigBrother_BuffWindow and BigBrother_BuffWindow:IsShown() then
		addon:ScheduleEvent(BuffWindow_Update,0.25)
	end
end

